package javaff.heuristics;

import java.math.BigDecimal;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javaff.data.Action;
import javaff.data.TotalOrderPlan;
import javaff.planning.STRIPSState;
import javaff.planning.State;

public class FFHeuristic implements Heuristic {
	
	public FFHeuristic() {}

	public BigDecimal getHValue(State state) {
		STRIPSState sstate = (STRIPSState) state;
		if (!(sstate.isRPCalculated())) {
			calculateRP(sstate);
		}
		return sstate.HValue();
	}
	
	public void calculateRP(STRIPSState sstate) {
		TotalOrderPlan relaxedPlan = (TotalOrderPlan) sstate.getRPG().getPlan(sstate);
		Set helpfulActions = new HashSet();
		if (!(relaxedPlan == null)) {
			sstate.setHValue(new BigDecimal(relaxedPlan.getPlanLength()));
			Iterator it = relaxedPlan.iterator();
			while (it.hasNext()) {
				Action a = (Action) it.next();
				if (sstate.getRPG().getLayer(a) == 0) {
					helpfulActions.add(a);
				}
			}
			sstate.setHelpfulActions(helpfulActions);
		} else {
			sstate.setHValue(javaff.JavaFF.MAX_DURATION);
		}
		sstate.setRPCalculated(true);
	}

}
