/************************************************************************
 * Strathclyde Planning Group,
 * Department of Computer and Information Sciences,
 * University of Strathclyde, Glasgow, UK
 * http://planning.cis.strath.ac.uk/
 * 
 * Copyright 2007, Keith Halsey
 * Copyright 2008, Andrew Coles and Amanda Smith
 *
 * (Questions/bug reports now to be sent to Andrew Coles)
 *
 * This file is part of JavaFF.
 * 
 * JavaFF is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * JavaFF is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JavaFF.  If not, see <http://www.gnu.org/licenses/>.
 * 
 ************************************************************************/

package javaff.planning;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javaff.data.Action;
import javaff.heuristics.Heuristic;

public class HelpfulFilter implements Filter
{
	
	public HelpfulFilter(Heuristic heuristic) {
		this.heuristic = heuristic;
	}

	private Heuristic heuristic;
	
	public Set getActions(State S)
	{
		STRIPSState SS = (STRIPSState) S;
		heuristic.getHValue(SS); // get the relaxed plan to the goal, to make sure helpful actions exist for S
		Set ns = new HashSet();
		if(SS.helpfulActions == null) {
			System.exit(1);
		}
		Iterator ait = SS.helpfulActions.iterator(); // iterate over helpful actions
		while (ait.hasNext())
		{
			Action a = (Action) ait.next();
			if (a.isApplicable(S)) ns.add(a); // and add them to the set to return if they're applicable
		}
		return ns;
	}
} 
